from __future__ import annotations

from typing import Any

import polars as pl


@pl.api.register_dataframe_namespace("tvtsplit")
class TVTsplit:
    def __init__(
        self,
        df: pl.DataFrame,
    ):
        self._df = df
        self.df_length = df.select(pl.len()).item()

    def train_length(self, params: dict[str, Any] | None = None) -> int:

        if params is None:
            params = {"shuffle": True, "seed": 0, "val_size": 0.25, "test_size": 0.25}

        return (
            self.df_length
            - int(self.df_length * params["val_size"])
            - int(self.df_length * params["test_size"])
        )

    def val_length(self, params: dict[str, Any] | None = None) -> int:

        if params is None:
            params = {"shuffle": True, "seed": 0, "val_size": 0.25, "test_size": 0.25}

        return int(self.df_length * params["val_size"])

    def test_length(self, params: dict[str, Any] | None = None) -> int:

        if params is None:
            params = {"shuffle": True, "seed": 0, "val_size": 0.25, "test_size": 0.25}

        return int(self.df_length * params["test_size"])

    def train(self, params: dict[str, Any] | None = None) -> pl.DataFrame:

        if params is None:
            params = {"shuffle": True, "seed": 0, "val_size": 0.25, "test_size": 0.25}

        df = (
            self._df.select(
                pl.all().shuffle(
                    seed=int(params["seed"]) if params["seed"] is not None else None
                )
            )
            if params["shuffle"]
            else self._df
        )

        return (
            df.with_row_index(name="n")
            .filter((pl.col("n") >= 0) & (pl.col("n") < self.train_length(params)))
            .drop("n")
        )

    def val(self, params: dict[str, Any] | None = None) -> pl.DataFrame:

        if params is None:
            params = {"shuffle": True, "seed": 0, "val_size": 0.25, "test_size": 0.25}

        df = (
            self._df.select(
                pl.all().shuffle(
                    seed=int(params["seed"]) if params["seed"] is not None else None
                )
            )
            if params["shuffle"]
            else self._df
        )

        return (
            df.with_row_index(name="n")
            .filter(
                (pl.col("n") >= self.train_length(params))
                & (pl.col("n") < self.train_length(params) + self.val_length(params))
            )
            .drop("n")
        )

    def test(self, params: dict[str, Any] | None = None) -> pl.DataFrame:

        if params is None:
            params = {"shuffle": True, "seed": 0, "val_size": 0.25, "test_size": 0.25}

        df = (
            self._df.select(
                pl.all().shuffle(
                    seed=int(params["seed"]) if params["seed"] is not None else None
                )
            )
            if params["shuffle"]
            else self._df
        )

        return (
            df.with_row_index(name="n")
            .filter(
                (pl.col("n") >= self.train_length(params) + self.val_length(params))
                & (pl.col("n") < self.df_length)
            )
            .drop("n")
        )

    def tvt(self, params: dict[str, Any] | None = None) -> dict[str, pl.DataFrame]:

        if params is None:
            params = {"shuffle": True, "seed": 0, "val_size": 0.25, "test_size": 0.25}

        return {
            "train": self.train(params),
            "val": self.val(params),
            "test": self.test(params),
        }



import polars as pl

import tvtsplit


def main():

    test_tvtsplit()


def test_tvtsplit():

    params = {"shuffle": False, "seed": 0, "val_size": 0.25, "test_size": 0.25}
    sample_df = pl.DataFrame(
        data=["aaa", "bbb", "ccc", "ddd", "eee", "fff"],
        schema=[("txt", pl.String)],
    )

    actual = sample_df.tvtsplit.train(params)["txt"].to_list()  # type: ignore[unresolved-attribute]
    expected = ["aaa", "bbb", "ccc", "ddd"]

    assert actual == expected


# I put a dummy function because the formatter deletes "import tvtsplit".
def dummy():
    tvtsplit()  # type: ignore[call-non-callable]


if __name__ == "__main__":
    main()
